<?php

/**
 * this class is used by the filepage extension to read its configuration.
 *
 */
class tx_filepage_config {
	
	/**
	 * the extension key
	 *
	 */
	const EXT= 'filepage';
	
	static protected $config_loaded= false;
	static protected $params;
	static protected $params_set;
	
	/**
	 * returns true if $value is considered to contain nothing, false otherwise.
	 * @see /?test/nothing
	 *
	 * @param unknown $value
	 *
	 * @return boolean
	 */
	static protected function nothing($value) {
		// ordered: according to their statistical frequency
		if (is_string($value)) {
			return strlen($value)==0;
		} elseif (is_null($value)) {
			return true;
		} elseif (is_int($value)) {
			return false;
		} elseif (is_array($value)) {
			return count($value)==0;
		} elseif (is_object($value)) {
			return false;
		} elseif (is_bool($value)) {
			return !$value;
		} elseif (is_float($value)) {
			return false;
		} elseif (is_resource($value)) {
			return false;
		} else {
			return true;
		}
	}
	
	static protected function ifnothing($first, $second) {
		$num= func_num_args();
		if ($num > 2) {
			for ($i= 0; $i < $num; $i++) {
				$arg= func_get_arg($i);
				if (!self::nothing($arg)) {
					return $arg;
				}
			}
		} else {
			if (self::nothing($first)) {
				return $second;
			} else {
				return $first;
			}
		}
	}
	
	static protected function load_config() {
		if (self::$config_loaded) {
			return;
		}
		
		$config= $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf'][self::EXT];
		if (strlen($config) > 0) {
			$config= unserialize($config);
			if ($config !== false) {
				foreach ($config as $key => $value) {
					self::$params[$key]= $value;
					self::$params_set[$key]= true;
				}
			}
		}
		self::$config_loaded= true;
	}
	
	/**
	 * returns true if the given parameter has been set. it returns true, even if the value has been set to NULL!
	 *
	 * @param string $param
	 * @return bool
	 */
	static public function is_set($param) {
		self::load_config();
		return self::$params_set[$param] == true;
	}
	
	static public function get($param, $default_if_value_is_nothing= null, $default_if_value_is_undefined= null) {
		self::load_config();
		if (self::is_set($param)) {
			$value= self::$params[$param];
			if ($default_if_value_is_nothing !== null) {
				return self::ifnothing($value, $default_if_value_is_nothing);
			} else {
				return $value;
			}
		} elseif ($default_if_value_is_undefined !== null) {
			return $default_if_value_is_undefined;
		} else {
			self::send_error('param undefined: "'.$param.'"');
		}
	}
	
	static public function set($param, $value) {
		self::load_config();
		self::$params[$param]= $value;
		self::$params_set[$param]= true;
	}
	
	static public function set_if_undefined($param, $value) {
		if (self::is_set($param)) {
			return;
		}
		self::set($param, $value);
	}
	
	static public function un_set($param) {
		self::load_config();
		unset(self::$params[$param]);
		self::$params_set[$param]= false;
	}
	
	/**
	 * writes a short generic message to the user, writes $msg to the php error log and terminates.
	 *
	 * @param string $msg
	 */
	static public function send_error($msg) {
		echo '<br><br>application error.';
		self::send_message($msg, E_USER_ERROR);
		exit; // application should already be terminated before this line
	}
	
	/**
	 * writes $msg as a warning to the php error log.
	 *
	 * @param string $msg
	 */
	static public function send_warning($msg) {
		self::send_message($msg, E_USER_WARNING);
	}
	
	/**
	 * writes $msg as a notice to the php error log
	 * 
	 * @param string $msg
	 */
	static public function send_notice($msg) {
		self::send_message($msg, E_USER_NOTICE);
	}
	
	/**
	 * writes $msg to the php error log.
	 *
	 * @param string $msg
	 */
	static protected function send_message($msg, $type) {
		$caller_stack= '';
		try {
			throw new Exception();
		} catch (Exception $e) {
			$caller_stack.= $msg."\n";
			$caller_stack.= $e->getTraceAsString();
		}
		if (strlen($caller_stack) <= 0) {
			$caller_stack= __CLASS__.'::'.__METHOD__.'('.$msg.')';
		}
		trigger_error($caller_stack, $type);
	}
	
	static public function dump() {
		$data= self::$params;
		ksort($data);
		echo '<hr><pre>';
		var_dump($data);
		echo '</pre><hr>';
	}
	
}

// this is a static class. therefore we have no xclass clause here. 

?>